import { type BaseEmbedControllerState } from "@toruslabs/base-controllers";
import { SafeEventEmitter, type WhiteLabelData } from "@web3auth/auth";
import { IPlugin, IWeb3AuthCore, PLUGIN_STATUS_TYPE, PluginConnectParams, SafeEventEmitterProvider } from "@web3auth/base";
import WsEmbed, { CtorArgs, WsEmbedParams } from "@web3auth/ws-embed";
type WsPluginEmbedParams = Omit<WsEmbedParams, "buildEnv" | "enableLogging" | "chainConfig" | "confirmationStrategy"> & {
    /**
     * Determines how to show confirmation screens
     * @defaultValue default
     *
     * default & auto-approve
     * - use auto-approve as default
     * - if wallet connect request use modal
     *
     * modal
     * - use modal always
     */
    confirmationStrategy?: Exclude<WsEmbedParams["confirmationStrategy"], "popup">;
};
export { BUTTON_POSITION, type BUTTON_POSITION_TYPE, CONFIRMATION_STRATEGY, type CONFIRMATION_STRATEGY_TYPE } from "@web3auth/ws-embed";
export declare class WalletServicesPlugin extends SafeEventEmitter implements IPlugin {
    name: "wallet-services";
    status: PLUGIN_STATUS_TYPE;
    readonly SUPPORTED_ADAPTERS: string[];
    readonly pluginNamespace: "multichain";
    wsEmbedInstance: WsEmbed;
    private provider;
    private web3auth;
    private isInitialized;
    private walletInitOptions;
    constructor(options?: {
        wsEmbedOpts?: Partial<CtorArgs>;
        walletInitOptions?: WsPluginEmbedParams;
    });
    get proxyProvider(): SafeEventEmitterProvider | null;
    initWithWeb3Auth(web3auth: IWeb3AuthCore, whiteLabel?: WhiteLabelData): Promise<void>;
    initWithProvider(): Promise<void>;
    connect({ sessionId, sessionNamespace }: PluginConnectParams): Promise<void>;
    showWalletConnectScanner(showWalletConnectParams?: BaseEmbedControllerState["showWalletConnect"]): Promise<void>;
    showCheckout(showCheckoutParams?: BaseEmbedControllerState["showCheckout"]): Promise<void>;
    showWalletUi(showWalletUiParams?: BaseEmbedControllerState["showWalletUi"]): Promise<void>;
    showSwap(showSwapParams?: BaseEmbedControllerState["showSwap"]): Promise<void>;
    cleanup(): Promise<void>;
    disconnect(): Promise<void>;
    private subscribeToWalletEvents;
    private subscribeToProviderEvents;
    private subscribeToWeb3AuthEvents;
    private sessionConfig;
    private walletServicesSessionConfig;
    private setSelectedAddress;
    private setChainID;
}
